@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
- * about.c
+ * about.c - the About dialog
  *
- *Copyright (C) 2004 Eugene Morenko(More) more@irpin.com
+ * Copyright (C) 2004 Eugene Morenko(More) more@irpin.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <gdk/gdk.h>
@@ -118,6 +118,8 @@ If you know of anyone missing from this list, please let us know\n\
 Thanks to all who helped making this software available.\n\
 ");
 	const gchar *TRANSLATORS = _("Current translators:\n\n\
+  Basque: Hizkuntza Politikarako Sailburuordetza\n\
+   <hizkpol@ej-gv.es>\n\n\
   Brazilian Portuguese: Anderson Rocha\n\
    <anderson@maxlinux.com.br>\n\n\
   Bulgarian: Peio Popov\n\
@@ -150,8 +152,10 @@ Thanks to all who helped making this software available.\n\
    <rupakov@jet.msk.su>\n\n\
   Serbian: Marko Milenović\n\
    <raven@sbb.co.yu>\n\n\
+  Slovak: Vladimir VASIL\n\
+   <vvasil@post.sk>\n\n\
   Spanish: Walter Oscar Echarri\n\
-   <wecharri@arnet.com.ar>\n\n\
+   <wecharri@gmail.com>\n\n\
   Swedish: David Smeringe\n\
    <david.smeringe@telia.com>\n\n\
   Tamil: Murugapandian Barathee\n\
