@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2004 Olivier Sessink
+ * Copyright (C) 2000-2005 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 /* #define DEBUG */
 
 #include <gtk/gtk.h>
@@ -323,6 +324,9 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 static GList *props_init_main(GList * config_rc)
 {
 /* these are used in the gtk-2 port already */
+#ifndef NOSPLASH
+	init_prop_integer   (&config_rc, &main_v->props.show_splash_screen, "show_splash_screen:", 1, TRUE);
+#endif /* #ifndef NOSPLASH */
 	init_prop_integer   (&config_rc, &main_v->props.show_quickbar_tip, "show_quickbar_tip:", 1, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.view_line_numbers, "view_line_numbers:", 1, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_hidden_files, "fb_show_hidden_f:", 0, TRUE);
@@ -333,12 +337,16 @@ static GList *props_init_main(GList * config_rc)
 	init_prop_string    (&config_rc, &main_v->props.filebrowser_dir_icon, "fb_dir_icon:", PKGDATADIR"icon_dir.png");
 	init_prop_string    (&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "courier 11");
 	init_prop_integer   (&config_rc, &main_v->props.editor_tab_width, "editor_tab_width:", 3, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.editor_indent_wspaces, "editor_indent_wspaces:", 0, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
 	init_prop_arraylist (&config_rc, &main_v->props.browsers, "browsers:", 2, TRUE);
 	init_prop_arraylist (&config_rc, &main_v->props.external_commands, "external_commands:", 2, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.highlight_num_lines_count, "highlight_num_lines_count:", 5, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1, TRUE);
+#ifdef HAVE_PCRE_UTF8
+	init_prop_integer   (&config_rc, &main_v->props.highlight_utf8, "highlight_utf8:", 0, TRUE);
+#endif /* HAVE_PCRE_UTF8 */
 	/* old type filetypes have a different count, they are converted below */
 	init_prop_arraylist (&config_rc, &main_v->props.filetypes, "filetypes:", 0, TRUE);
 	init_prop_integer   (&config_rc, &main_v->props.numcharsforfiletype, "numcharsforfiletype:", 200, TRUE);
@@ -744,17 +752,30 @@ static void rcfile_custom_menu_load_all(gboolean full_reset, gchar *defaultfile)
 	}
 }
 
+/*
+ - If LC_ALL is set and non-null, follow it.
+ - Else if LC_MESSAGES is set and non-null, follow it.
+ - Else if LANG is set and non-null, follow it.
+*/
 void rcfile_parse_custom_menu(gboolean full_reset, gboolean load_new) {
 	gchar *defaultfile, *langdefaultfile1=NULL, *langdefaultfile2=NULL, *tmp;
+	const gchar *tmp2;
 	DEBUG_MSG("rcfile_parse_custom_menu, started\n");
 
+	tmp2 = g_getenv("LC_ALL");
+	if (tmp2 == NULL) {
+		tmp2 = g_getenv("LC_MESSAGES");
+		if (tmp2 == NULL) {
 #ifdef PLATFORM_DARWIN
-	tmp = g_strdup(g_getenv("LANGUAGE"));
+			tmp2 = g_getenv("LANGUAGE");
 #else
-	tmp = g_strdup(g_getenv("LANG"));
+			tmp2 = g_getenv("LANG");
 #endif
+		}
+	}
+	tmp = g_strdup(tmp2);
 	DEBUG_MSG("rcfile_parse_custom_menu, Language is: %s", tmp);
-	if (tmp) {
+	if (tmp && strlen(tmp)>0) {
 		tmp = trunc_on_char(tmp, '.');
 		tmp = trunc_on_char(tmp, '@');
 		langdefaultfile1 = g_strconcat(PKGDATADIR"custom_menu.",tmp,".default", NULL);
